11.6   os32: fs(9)

Il file kernel/fs.h [12.5] descrive cio` che serve per la gestione del
file system, che per os32 corrisponde al tipo Minix 1, assieme ai socket,
essendo questi assimilati ai descrittori di file.

La gestione del file system e dei socket, a livello complessivo di sistema, e`
suddivisa in quattro aspetti principali: super blocco, inode, file e socket.
Per ognuno di questi e` prevista una tabella (di super blocchi, di inode, di
file e di socket). Seguono delle figure e i listati che descrivono
l'organizzazione di queste tabelle.

Figura 2.95. Struttura del tipo sb_t, corrispondente agli elementi dell'array
<sb_table[]>.

 typedef struct sb        sb_t;

 struct sb {
   uint16_t  inodes;
   uint16_t  zones;
   uint16_t  map_inode_blocks;
   uint16_t  map_zone_blocks;
   uint16_t  first_data_zone;
   uint16_t  log2_size_zone;
   uint32_t  max_file_size;
   uint16_t  magic_number;
   //-------------------------
   dev_t     device;
   inode_t  *inode_mounted_on;
   blksize_t blksize;
   int       options;
   uint16_t  map_inode[SB_MAP_INODE_SIZE];
   uint16_t  map_zone[SB_MAP_ZONE_SIZE];
   char      changed;
 };

Figura 2.99. Struttura del tipo inode_t, corrispondente agli elementi dell'
array <inode_table[]>.

 typedef struct inode     inode_t;

 struct inode {
     mode_t        mode;
     uid_t         uid;
     ssize_t       size;
     time_t        time;
     uint8_t       gid;
     uint8_t       links;
     zno_t         direct[7];
     zno_t         indirect1;
     zno_t         indirect2;
     //------------------------
     sb_t         *sb;
     ino_t         ino;
     sb_t         *sb_attached;
     blkcnt_t      blkcnt;
     unsigned char references;
     char          changed  : 1,
                   pipe_dir : 1;
     unsigned char pipe_off_read;
     unsigned char pipe_off_write;
     unsigned char pipe_ref_read;
     unsigned char pipe_ref_write;
 };

Figura 2.106. Struttura del tipo file_t, corrispondente agli elementi dell'
array <file_table[]>.

 typedef struct file file_t;

 struct file {
     int       references;
     off_t     offset;
     int       oflags;
     inode_t  *inode;
     sock_t   *sock;
 };

Figura 2.109. Struttura del tipo fd_t, con cui si costituiscono gli elementi
delle tabelle dei descrittori di file, una per ogni processo.

 typedef struct fd fd_t;
 struct fd {
     int     fl_flags;
     int     fd_flags;
     file_t *file;
 };

Listato 2.129. Struttura di ogni elemento della tabella dei socket.

struct sock
{
  int            family;
  int            type;
  int            protocol;
  h_addr_t       laddr;         // indirizzo locale
  h_port_t       lport;         // porta locale
  h_addr_t       raddr;         // indirizzo remoto
  h_port_t       rport;         // porta remota
  struct {
    clock_t      clock[IP_MAX_PACKETS];
  } read;
  uint8_t        active      : 1, // socket utilizzato?
                 unreach_net : 1, // rete irraggiungibile?
                 unreach_host: 1, // destinazione irraggiungibile?
                 unreach_prot: 1, // protocollo irraggiungibile?
                 unreach_port: 1; // porta irraggiungibile?
  struct
  {
    uint16_t     conn        : 4, // stato della connessione
                 can_write   : 1, // si puo` scrivere in `send_data[]'?
                 can_read    : 1, // si puo` leggere a partire da
                                  //   `*recv_index'?
                 can_send    : 1, // si possono inviare dati?
                 can_recv    : 1, // si possono ricevere dati?
                 send_closed : 1, // il canale di trasmissione
                                  //   e` chiuso?
                 recv_closed : 1; // il canale di ricezione e` chiuso?
    //
    uint32_t     lsq[16];       // array della sequenza locale
    uint32_t     lsq_ack;       // numero di sequenza in attesa
                                //   di conferma
    uint32_t     rsq[16];       // array della sequenza remota
    uint8_t      lsqi      : 4, // indice dell'array della sequenza
                                //   locale
                 rsqi      : 4; // indice dell'array della sequenza
                                //   remota
    //
    clock_t      clock;         // istante dell'ultima trasmissione
    //
    uint8_t      send_data[TCP_MSS - sizeof (struct tcphdr)];
                                // dati da trasmettere
    size_t       send_size;     // dimensione dei dati da trasmettere
    int          send_flags;
    uint8_t      recv_data[TCP_MAX_DATA_SIZE];  // dati ricevuti
    size_t       recv_size;     // dimensione dei dati
                                //   ricevuti
    uint8_t     *recv_index;    // indice per la lettura dei dati
                                //   ricevuti
    pid_t        listen_pid;    // processo in ascolto della porta
                                //   locale, in attesa di connessioni
    int          listen_max;    // numero massimo di richieste di
                                //   connessione accettabili
    int          listen_queue[SOCK_MAX_QUEUE];
                                // descrittori di connessioni
                                //   realizzate
  } tcp;
};

